import numpy as np
import matplotlib.pyplot as plt

from astropy.modeling.blackbody import blackbody_lambda, blackbody_nu
from astropy.modeling.models import BlackBody
import astropy.units as u

wavelengths = np.logspace(np.log10(1000), np.log10(3e4), num=1000) * u.AA

# blackbody parameters
temperature = 10000 * u.K

# provides the results in ergs/(cm^2 Hz s sr)
spectrum_nu = blackbody_nu(wavelengths, temperature)

# provides the result in ergs/(cm^2 A s sr)
spectrum_lam = blackbody_lambda(wavelengths, temperature)

# default results from BlackBody same units as blackbody_nu
bb_nu = BlackBody(temperature)

# use scale to change results units for Blackbody to match blackbody_lambda
bb_lam = BlackBody(temperature, scale=1.0 * u.erg / (u.cm ** 2 * u.AA * u.s * u.sr))

# plot the nu and lam results from both methods
fig, ax = plt.subplots(ncols=2, figsize=(8.0, 4.0))

# lambda forms
ax[0].plot(wavelengths, spectrum_lam, label="blackbody_lambda", linewidth=6, alpha=0.5)
ax[0].plot(wavelengths, bb_lam(wavelengths), label="BlackBody")

# nu forms
ax[1].plot(wavelengths, spectrum_nu, label="blackbody_nu", linewidth=6, alpha=0.5)
ax[1].plot(wavelengths, bb_nu(wavelengths), label="BlackBody")

ax[0].set_xlabel(r"$\lambda$ [{}]".format(wavelengths.unit))
ax[0].set_ylabel(r"$Flux$ [{}]".format(spectrum_lam.unit))
ax[1].set_xlabel(r"$\lambda$ [{}]".format(wavelengths.unit))
ax[1].set_ylabel(r"$Flux$ [{}]".format(spectrum_nu.unit))
ax[0].set_xscale("log")
ax[0].set_yscale("log")
ax[1].set_xscale("log")
ax[1].set_yscale("log")
ax[0].legend(loc="best")
ax[1].legend(loc="best")

plt.tight_layout()
plt.show()