import numpy as np
import matplotlib.pyplot as plt
from scipy import stats
from astropy.modeling import models, fitting

# Set up the shape of the image and create the time axis
depth,width,height=10,3,4 # Time is along the depth axis
t = np.arange(depth, dtype=np.float64)*10.  # e.g. readouts every 10 seconds

# Make up a flux in each pixel
fluxes = np.arange(1.*width*height).reshape(height, width)
# Create the ramps by integrating the fluxes along the time steps
image = fluxes[np.newaxis, :, :] * t[:, np.newaxis, np.newaxis]
# Add some Gaussian noise to each sample
image += stats.norm.rvs(0., image*0.05, size=image.shape)  # Add noise

# Create the models and the fitter
N = width * height # This is how many instances we need
line = models.Polynomial1D(degree=1, n_models=N)
fit = fitting.LinearLSQFitter()

# We need to get the data to be fit into the right shape
# In this case, the time axis can be one dimensional.
# The fluxes have to be organized into an array
# that is of shape `(width*height, depth)`
# i.e we are reshaping to flatten last two axes and
# transposing to put them first.
pixels = image.reshape((depth, width*height))
y = pixels.T

# Fit the model. It does the looping over the N models implicitly
new_model = fit(line, x=t, y=y)

# Fill an array with values computed from the best fit and reshape it to match the original
best_fit = new_model(t, model_set_axis=False).T.reshape((depth, height, width))


# Plot the fit along a couple of pixels
def plotramp(t, image, best_fit, row, col):
    plt.plot(t, image[:, row, col], '.', label=f'data pixel {row},{col}')
    plt.plot(t, best_fit[:, row, col], '-', label=f'fit to pixel {row},{col}')
    plt.xlabel('Time')
    plt.ylabel('Counts')
    plt.legend(loc='upper left')


plt.figure(figsize=(10, 5))
plotramp(t, image, best_fit, 1, 1)
plotramp(t, image, best_fit, 3, 2)
plt.show()